@@ -1,6 +1,8 @@
-#!/bin/env perl 
+#!perl -w
 #!d:\perl\bin\perl.exe 
 
+# -- SOAP::Lite -- soaplite.com -- Copyright (C) 2001 Paul Kulchenko --
+
 use strict;
 use Benchmark;
 use SOAP::Lite on_fault => sub {my($soap, $res) = @_; die ref $res ? $res->faultdetail : $soap->transport->status, "\n"};
@@ -28,14 +30,12 @@ my $testnum = 3;
 my $testtime = 5;
 my %result;
 
-print STDERR <<DISCLAYMER;
+print STDERR <<DISCLAIMER;
 
-This test should be used only for comparison different Perl implementations 
-running in your environment. 
+This test should be used only for comparison different Perl server
+implementations running in your environment. 
 
-'tcpip' tests use one-way calls (twice less de/serializations than others)
-
-DISCLAYMER
+DISCLAIMER
 
 print STDERR "All tests may take up to ", %dests * %tests * $testnum * $testtime, " sec\n";
 
